//
//  CeBitmapView.m
//  BitmapEffets
//
// Code source d'origine par Renaud Pradenc.
// Ce source a été créé pour la présentation de Cocoa Heads Paris (oct 2010).
// Vous êtes libres de le réutiliser, de le modifier et de le redistribuer.
// Soyez sympas: conservez simplement ce texte de présentation.



#import "CeBitmapView.h"


@implementation CeBitmapView


// MARK: Properties
@synthesize bitmap;
- (void) setBitmap:(CeImage *)newBitmap
{
	[bitmap release];
	bitmap = [newBitmap retain];
	
	[self setNeedsDisplay:YES];
}

// MARK: Drawing
- (void) drawRect:(NSRect)dirtyRect
{
	// 18% grey background
	[[NSColor colorWithCalibratedWhite:0.18 alpha:1.0] setFill];
	[NSBezierPath fillRect:dirtyRect];
	
	// Draw the bitmap
	CGRect destRect = [CeBitmapView _rectOfSize:[bitmap size] centredInRect:NSRectToCGRect([self bounds])];
	destRect = CGRectInset(destRect, 4, 4);
	[bitmap drawInRect:destRect];	
}


// Renvoyer un rectangle de mêmes proportions que 'size' et centré dans boundsRect.
+ (CGRect) _rectOfSize:(CGSize)size centredInRect:(CGRect)boundsRect
{
	float sourceProportions = size.width / size.height;
	float boundsProportions = boundsRect.size.width /boundsRect.size.height;
	
	float scale;
	if(sourceProportions > boundsProportions)	// Will not fit in width
	{
		scale = boundsRect.size.width / size.width;
	}
	else 	// Equal proportions, or will not fit in height
	{
		scale = boundsRect.size.height / size.height;
	}

	CGRect destRect;
	destRect.size.width = size.width * scale;
	destRect.size.height = size.height * scale;
	destRect.origin.x = boundsRect.origin.x + (boundsRect.size.width-destRect.size.width)/2.0;
	destRect.origin.y = boundsRect.origin.y + (boundsRect.size.height-destRect.size.height)/2.0;
	
	return destRect;
}


@end
