//
//  CeConvolutionMatrix.m
//  BitmapEffets
//
// Code source d'origine par Renaud Pradenc.
// Ce source a été créé pour la présentation de Cocoa Heads Paris (oct 2010).
// Vous êtes libres de le réutiliser, de le modifier et de le redistribuer.
// Soyez sympas: conservez simplement ce texte de présentation.



#import "CeConvolutionMatrix.h"


@implementation CeConvolutionMatrix

- (id) init
{
	if(self = [super init])
	{
		// Bas-relief
		nw = 2.0;	n =  1.0;	ne =  0.0;
		w  = 1.0;	c =  1.0;	e  = -1.0;
		sw = 0.0;	s = -1.0;	se = -2.0;
	}
	
	return self;
}


- (void) applyEffectDestBitmap:(PixelRGBx *)destBitmapPtr 
				  sourceBitmap:(PixelRGBx *)sourceBitmapPtr 
						 width:(NSUInteger)width 
						height:(NSUInteger)height
{
	PixelRGBx *sourcePixel;
	PixelRGBx *destPixel;
	
	NSUInteger x;
	NSUInteger y;

	sourcePixel = sourceBitmapPtr;
	destPixel = destBitmapPtr;
	
	for(y = 0; y < height; y++)
	{
		for(x = 0; x < width; x++)
		{
			// First/last column/row are not processed to simplify the problem of the edges
			if( (y > 0) && (y < height-1) && (x>0) && (x < width-1))
			{
				// Get surrounding pixels
				PixelRGBx *wPixel = sourcePixel-1;
				PixelRGBx *ePixel = sourcePixel+1;
				PixelRGBx *nPixel = sourcePixel-width;
				PixelRGBx *nwPixel = nPixel-1;
				PixelRGBx *nePixel = nPixel+1;
				PixelRGBx *sPixel = sourcePixel+width;
				PixelRGBx *swPixel = sPixel-1;
				PixelRGBx *sePixel = sPixel+1;
				
				// Apply the matrix
				float coeffSum = nw + n + ne
								+ w + c + e
								+ sw + s + se;
				
				float r = (nw*nwPixel->comp.r + n*nPixel->comp.r + ne*nePixel->comp.r
						   + w*wPixel->comp.r + c*sourcePixel->comp.r + e*ePixel->comp.r
						   + sw*swPixel->comp.r + s*sPixel->comp.r + se*sePixel->comp.r) / coeffSum;
				
				float g = (nw*nwPixel->comp.g + n*nPixel->comp.g + ne*nePixel->comp.g
						   + w*wPixel->comp.g + c*sourcePixel->comp.g + e*ePixel->comp.g
						   + sw*swPixel->comp.g + s*sPixel->comp.g + se*sePixel->comp.g) / coeffSum;
				
				float b = (nw*nwPixel->comp.b + n*nPixel->comp.b + ne*nePixel->comp.b
						   + w*wPixel->comp.b + c*sourcePixel->comp.b + e*ePixel->comp.b
						   + sw*swPixel->comp.b + s*sPixel->comp.b + se*sePixel->comp.b) / coeffSum;
				
				// Prevent values <0 and >255
				if(r < 0)	r = 0;
				if(r > 255)	r = 255;
				
				if(g < 0)	g = 0;
				if(g > 255)	g = 255;
				
				if(b < 0)	b = 0;
				if(b > 255)	b = 255;
				
				// Store the pixel
				destPixel->comp.r = r;
				destPixel->comp.g = g;
				destPixel->comp.b = b;
			}
			sourcePixel++;
			destPixel++;
		}
	}
}



@end
