//
//  CeEffect.m
//  BitmapEffets
//
// Code source d'origine par Renaud Pradenc.
// Ce source a été créé pour la présentation de Cocoa Heads Paris (oct 2010).
// Vous êtes libres de le réutiliser, de le modifier et de le redistribuer.
// Soyez sympas: conservez simplement ce texte de présentation.



#import "CeEffectChain.h"




@implementation CeEffectChain

// MARK: Class Mgt
- (id) init
{
	if(self = [super init])
	{
		effects = [[NSMutableArray alloc] init];
	}
	
	return self;
}


- (void) dealloc
{
	[self setSourceImage:nil];
	[effects release];
	
	[super dealloc];
}


// MARK: Properties
@synthesize sourceImage;
- (void) setSourceImage:(CeImage *)image
{
	// Free previous images / context
	[sourceImage release];
	[[self class] _releaseCGBitmapContext:sourceContext];
	
	[destImage release];
	destImage = nil;
	
	// Create the new context
	if(image)
	{
		sourceImage = [image retain];
		
		NSUInteger width = [sourceImage size].width;
		NSUInteger height = [sourceImage size].height;
		sourceContext = [[self class] _createCGBitmapContextWithWidth:width height:height];
	
		// Draw the source image in the context
		[sourceImage drawInCGContext:sourceContext rect:CGRectMake(0, 0, width, height)];
	}
	
}

- (CeImage *) destImage
{
	if(sourceImage == nil)
		return nil;
	
	[self _applyEffects];
	
	return destImage;
}




// MARK: Effects
- (void) addEffect:(CeEffect *)effect
{
	[effects addObject:effect];	
}

- (void) _applyEffects
{	
	NSUInteger width = CGBitmapContextGetWidth(sourceContext);
	NSUInteger height = CGBitmapContextGetHeight(sourceContext);
	
	PixelRGBx *sourceBitmap = CGBitmapContextGetData(sourceContext);
	
	// Apply the effects
	CGContextRef destContext;
	CGContextRef oldDestContext = nil;
	BOOL atLeastOneEffectEnabled = NO;
	for(CeEffect *effect in effects)
	{
		if(effect.enabled)
		{
			atLeastOneEffectEnabled = YES;
			
			destContext = [[self class] _createCGBitmapContextWithWidth:width height:height];
			PixelRGBx *destBitmap = CGBitmapContextGetData(destContext);
			[effect applyEffectDestBitmap:destBitmap sourceBitmap:sourceBitmap width:width height:height];
			
			if(oldDestContext)
				[[self class] _releaseCGBitmapContext:oldDestContext];
			
			oldDestContext = destContext;
			sourceBitmap = destBitmap;
		}
	}

	
	if(atLeastOneEffectEnabled)
	{
		// Create the dest image
		destImage = [[CeImage alloc] initWithCGBitmapContext:destContext];
		[[self class] _releaseCGBitmapContext:destContext];
	}
	
	else 
	{
		// The destImage = the source image
		destImage = [sourceImage retain];
	}

}


// MARK: Bitmaps and contexts handling

+ (CGContextRef) _createCGBitmapContextWithWidth:(NSUInteger)width height:(NSUInteger)height
{
	CGColorSpaceRef colorspace = CGColorSpaceCreateDeviceRGB();
	NSAssert(colorspace != NULL, @"Impossible de créer le colorspace");
	
	void *bitmap = malloc(width * height * sizeof(PixelRGBx));
	
	CGContextRef context = CGBitmapContextCreate(bitmap, 							// data
												 width,								// width
												 height, 							// height,
												 8, 								// bitsPerComponent
												 width*sizeof(PixelRGBx),			// bytesPerRow
												 colorspace,						// colorspace
												 kCGImageAlphaNoneSkipLast);		// bitmapInfo
	NSAssert(context != NULL, @"Impossible de créer le CGBitmapContext");
	CGColorSpaceRelease(colorspace);
	
	return context;
}

+ (void) _releaseCGBitmapContext:(CGContextRef)context
{
	if(context)
	{
		free(CGBitmapContextGetData(context));	// Free the memory taken by the bitmap
		CGContextRelease(context);
	}
}



@end
